@@ -1,7 +1,6 @@
 """OpenPifPaf web demo server process."""
 
 import argparse
-import json
 import logging
 import os
 import random
@@ -14,105 +13,38 @@ import torch
 import tornado
 import tornado.autoreload
 import tornado.httpclient
-from tornado.web import RequestHandler
 
 import openpifpaf
 
 from .processor import Processor
-from . import __version__ as VERSION
+from . import __version__
+from . import handlers
+from .signal import Signal
+
+LOG = logging.getLogger(__name__)
 
 
-# pylint: disable=abstract-method
-class PostHandler(RequestHandler):
-    def initialize(self, processor, *, demo_password):
-        self.processor = processor  # pylint: disable=attribute-defined-outside-init
-        self.demo_password = demo_password  # pylint: disable=attribute-defined-outside-init
-
-    def set_default_headers(self):
-        self.set_header('Access-Control-Allow-Origin', '*')
-        self.set_header('Access-Control-Allow-Headers',
-                        'Content-Type, Access-Control-Allow-Headers')
-
-    def post(self):  # pylint: disable=arguments-differ
-        self.set_default_headers()
-
-        image = self.get_argument('image', None)
-        if image is None:
-            image = json.loads(self.request.body).get('image', None)
-        if image is None:
-            self.write('no image provided')
-            return
-
-        resize = True
-        if self.demo_password:
-            if self.get_argument('pw', None) != self.demo_password:
-                self.write(json.dumps({'error': 'demo in progress'}))
-                return
-            resize = False
-        keypoint_sets, scores, width_height = self.processor.single_image(image, resize=resize)
-        keypoint_sets = [{
-            'coordinates': keypoints.tolist(),
-            'detection_id': i,
-            'score': score,
-            'width_height': width_height,
-        } for i, (keypoints, score) in enumerate(zip(keypoint_sets, scores))]
-        self.write(json.dumps(keypoint_sets))
-
-    def options(self):
-        self.set_default_headers()
-        self.set_status(204)
-        self.finish()
-
-
-class RenderTemplate(tornado.web.RequestHandler):
-    def initialize(self, template_name, **info):
-        self.template_name = template_name  # pylint: disable=attribute-defined-outside-init
-        self.info = info  # pylint: disable=attribute-defined-outside-init
-
-    def get(self):
-        self.render(self.template_name, **self.info)
-
-    def head(self):
-        pass
-
-
-class Index(tornado.web.RequestHandler):
-    def initialize(self, template_name, demo_password, **info):
-        self.template_name = template_name  # pylint: disable=attribute-defined-outside-init
-        self.demo_password = demo_password  # pylint: disable=attribute-defined-outside-init
-        self.info = info  # pylint: disable=attribute-defined-outside-init
-
-    def get(self):
-        password = self.get_argument('pw', default=None)
-        if self.demo_password and password == self.demo_password:
-            self.info['width_height'] = (801, 601)
-        elif self.demo_password:
-            self.write('Demo currently in progress. Check back later.')
-            return
-        self.render(self.template_name, **self.info)
-
-    def head(self):
-        pass
-
-
-async def grep_static(dest, url='http://127.0.0.1:5000'):
+async def grep_static(dest, url='http://127.0.0.1:5000/client.html', encoding='utf8'):
     http_client = tornado.httpclient.AsyncHTTPClient()
     response = await http_client.fetch(url)
     out = response.body.decode()
-    with open(dest, 'w') as f:
+    with open(dest, 'w', encoding=encoding) as f:
         f.write(out)
     http_client.close()
+    sys.exit(0)
 
 
 def cli():
+    openpifpaf.plugin.register()
+
     parser = argparse.ArgumentParser(
         description=__doc__,
         formatter_class=argparse.ArgumentDefaultsHelpFormatter,
     )
 
-    openpifpaf.decoder.cli(parser, force_complete_pose=False,
-                           instance_threshold=0.1, seed_threshold=0.5)
-    openpifpaf.network.cli(parser)
+    openpifpaf.decoder.cli(parser)
+    openpifpaf.logger.cli(parser)
+    openpifpaf.network.Factory.cli(parser)
 
     parser.add_argument('--disable-cuda', action='store_true',
                         help='disable CUDA')
@@ -123,8 +55,6 @@ def cli():
                         help='directory in which to create a static version of this page')
     parser.add_argument('--demo-password', default=None,
                         help='password that allows better performance for a demo')
-    parser.add_argument('--debug', default=False, action='store_true',
-                        help='debug messages and autoreload')
     parser.add_argument('--google-analytics',
                         help='provide a google analytics id to inject analytics code')
 
@@ -148,17 +78,17 @@ def cli():
 
     args = parser.parse_args()
 
-    # log
-    logging.basicConfig(level=logging.INFO if not args.debug else logging.DEBUG)
-
     # configure
-    openpifpaf.network.configure(args)
+    openpifpaf.logger.configure(args, LOG)
+    openpifpaf.network.Factory.configure(args)
 
     # config
-    logging.debug('host=%s, port=%d', args.host, args.port)
-    logging.debug('Python %s, OpenPifPafWebDemo %s', sys.version, VERSION)
+    LOG.debug('host=%s, port=%d', args.host, args.port)
+    LOG.debug('Python %s, OpenPifPafWebDemo %s', sys.version, __version__)
     if args.host in ('localhost', '127.0.0.1'):
-        logging.info('Open http://%s:%d in a web browser.', args.host, args.port)
+        LOG.info('Open http://%s:%d in a web browser.', args.host, args.port)
+    if args.host != '0.0.0.0':
+        LOG.info('Access is restricted by IP address. Use --host=0.0.0.0 to allow all.')
 
     # add args.device
     args.device = torch.device('cpu')
@@ -168,11 +98,18 @@ def cli():
     return args
 
 
+class Application(tornado.web.Application):
+    def __init__(self, handler_list, *, processor, signal, **settings):
+        self.processor = processor
+        self.signal = signal
+        super().__init__(handler_list, **settings)
+
+
 def main():
     args = cli()
     width_height = (int(640 * args.resolution // 16) * 16 + 1,
                     int(480 * args.resolution // 16) * 16 + 1)
-    logging.debug('target width and height = %s', width_height)
+    LOG.debug('target width and height = %s', width_height)
     processor_singleton = Processor(width_height, args)
 
     static_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'static')
@@ -186,17 +123,17 @@ def main():
     tornado.autoreload.watch('openpifpafwebdemo/static/frontend.js')
     tornado.autoreload.watch('openpifpafwebdemo/static/clientside.js')
 
-    if args.debug:
+    if LOG.getEffectiveLevel() == logging.DEBUG:
         version = '{}-{}'.format(
-            VERSION,
+            __version__,
             ''.join(random.choices(string.ascii_lowercase + string.digits, k=4))
         )
     else:
-        version = VERSION
+        version = __version__
 
-    app = tornado.web.Application(
+    app = Application(
         [
-            (r'/', Index, {
+            (r'/', handlers.Index, {
                 'template_name': 'index.html',
                 'demo_password': args.demo_password,
 
@@ -207,7 +144,7 @@ def main():
                 'google_analytics': args.google_analytics,
                 'width_height': width_height,
             }),
-            (r'/client.html', RenderTemplate, {
+            (r'/client.html', handlers.RenderTemplate, {
                 'template_name': 'client.html',
                 'title': 'OpenPifPafWebDemo Serverless',
                 'description': 'Interactive web browser based demo of OpenPifPaf.',
@@ -224,13 +161,14 @@ def main():
             (r'/(favicon\.ico)', tornado.web.StaticFileHandler, {
                 'path': os.path.join(static_path, 'favicon.ico'),
             }),
-            (r'/process', PostHandler, {
-                'processor': processor_singleton,
-                'demo_password': args.demo_password,
-            }),
+            (r'/v1/feed', handlers.Feed),
+            (r'/v1/human-poses', handlers.HumanPoses),
         ],
         debug=args.debug,
+        processor=processor_singleton,
+        signal=Signal(),  # can also by Redis PubSub
         static_path=static_path,
+        template_path=os.path.dirname(__file__),
     )
     app.listen(args.port, args.host)
 
@@ -247,7 +185,7 @@ def main():
                 'keyfile': os.path.join(module_dir, 'test', 'test.key'),
             }
 
-        logging.info('Open https://%s:%d in a web browser.', args.host, args.ssl_port)
+        LOG.info('Open https://%s:%d in a web browser.', args.host, args.ssl_port)
         app.listen(args.ssl_port, ssl_options=ssl_ctx)
 
     try:
